/*
 * Dan Armendariz
 * danallan@cs.harvard.edu
 *
 * More dictionaries, higher-order functions (HOFs)
 *
 * Run this code in an Xcode Playground by double-clicking its file in Finder.
 */

class Assessment
{
    var name: String
    var grades: [Int] = []

    // a special initialization function
    init(title: String)
    {
        self.name = title
    }
    
    // a method that accepts a grade and appends it to our internal representation of grades
    func addGrade(score: Int)
    {
        self.grades.append(score)
    }

    // the "..." syntax allows us to accept any number of parameters as an array
    func addGrades(scores: Int...)
    {
        // map over each score, adding it to our list of scores
        scores.map(self.addGrade)
    }
    
    // Return the average of a list of Ints as an optional Double
    func average() -> Double?
    {
        // return nil if we cannot compute the average
        if self.grades.isEmpty
        {
            return nil
        }
        
        // use a higher-order function to sum all values
        let sum = self.grades.reduce(0, +)

        return Double(sum) / Double(self.grades.count)
    }

}

// instantiate two Assessment objects
let quizzes = Assessment(title: "Quizzes")
let psets = Assessment(title: "Problem Sets")

// we can access properties outside of the class (usually)
quizzes.name
psets.name

// add some grades
quizzes.addGrade(100)
quizzes.addGrade(80)

// another property allows us to add several grades at once
psets.addGrades(80, 20, 100, 65)

// safely ensure we received data
if let avg = quizzes.average()
{
    avg
}
else
{
    println("No average for quizzes")
}

// or we can force-unwrap an optional with !
psets.average()!