#!/usr/bin/xcrun swift

/*
 * Dan Armendariz
 * danallan@cs.harvard.edu
 *
 * Control flow, functions, and optionals
 */

// Define functions with "func" keyword, specify parameters in parentheses
func printGradeCount(gradebook: [Int])
{
    if (gradebook.isEmpty)
    {
        println("The gradebook has no grades.")
    }
    else
    {
        println("The gradebook has \(gradebook.count) grades.")
    }
}

// Define a function's return type after an arrow
func average(gradebook: [Int]) -> Double
{
    // the parens are optional in if statements, but curly braces are not
    if gradebook.isEmpty
    {
        return 0.0
    }

    var sum = 0
    for grade in gradebook
    {
        sum += grade
    }
    return Double(sum) / Double(gradebook.count)
}

// define an (empty) mutable ARRAY of grades
var grades: [Int] = []

// Process.arguments is of type [String] and contains the runtime args.
// This for loop syntax iterates over all objects in an array!
for argument in Process.arguments
{
    // optionals: String.toInt() returns an "Int?" type (optional Int)
    // if it has a value (because conversion was successful) then it is
    // assigned to grade, else nil
    if let grade = argument.toInt()?
    {
        println("Accepting \(argument) as an integer")
        grades.append(grade)
    }
    else
    {
        // use backslash as an escape character
        println("Rejecting \"\(argument)\" Is it an integer?")
    }
}

// print the grade count
printGradeCount(grades)

// print the average
println("The gradebook average: \(average(grades))")
