#!/usr/bin/xcrun swift

/*
 * Dan Armendariz
 * danallan@cs.harvard.edu
 *
 * Dictionaries
 */

// needed for exit()
import Foundation

func printUsage()
{
    println("USAGE: ./6.swift ASSESSMENT_NAME:GRADE")
    println("Every ASSESSMENT_NAME must be unique.")
    exit(0)
}

// Define functions with "func" keyword, specify parameters in parentheses
func printGradeCount(gradebook: [String:Int])
{
    if (gradebook.isEmpty)
    {
        println("The gradebook has no grades.")
    }
    else
    {
        // Concatenate all of the names of assessments in gradebook with a comma
        let listOfNames = ", ".join(gradebook.keys)
        println("The gradebook has \(gradebook.count) grades: \(listOfNames)")
    }
}

// Define a function's return type after an arrow
func average(gradebook: [String:Int]) -> Double
{
    // the parens are optional in if statements, but curly braces are not
    if gradebook.isEmpty
    {
        return 0.0
    }

    var sum = 0
    for grade in gradebook.values
    {
        sum += grade
    }
    return Double(sum) / Double(gradebook.count)
}

// define an (empty) mutable DICTIONARY of grades
var grades: [String:Int] = [:]

// starting at index 1 to skip first argument (which is this file name)
for index in 1 ..< Process.arguments.count
{
    // split the argument at ":" characters into an array
    let components = split(Process.arguments[index], { $0 == ":" })

    // we expect only two components out of the split
    if components.count != 2
    {
        printUsage()
    }

    // fetch the name of the assessment from the array
    let name = components[0]

    // attempt to fetch the grade by converting to an Int
    if let grade = components[1].toInt()?
    {
        // successfully converted! let's see if the name already exists in dict
        if let existing = grades[name]
        {
            // duplicate grade names
            printUsage()
        }
        else
        {
            // new grade! enter it into the dictionary
            grades[name] = grade
        }
    }
    else
    {
        // unable to convert the grade to an Int
        printUsage()
    }
}

// we need to operate on some grades
if grades.isEmpty
{
    printUsage()
}

// print the grade count
printGradeCount(grades)

// print the average
println("The gradebook average: \(average(grades))")
